import json
import logging
from itertools import groupby
from typing import Dict, List, Union

import cv2
import numpy as np
import pandas as pd
from PIL import Image

from disclosure_extractor.data_processing import ocr_slice, clean_stock_names
from disclosure_extractor.image_processing import (
    find_redactions,
    load_template,
)


def box_extraction(page):

    open_cv_image = np.array(page)
    img = open_cv_image[:, :, ::-1].copy()
    img = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    (thresh, img_bin) = cv2.threshold(
        img, 128, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU
    )
    img_bin = 255 - img_bin

    # Defining a kernel length
    kernel_length = np.array(img).shape[1] // 200

    # A verticle kernel of (1 X kernel_length), which will
    # detect all the verticle lines from the image.
    verticle_kernel = cv2.getStructuringElement(
        cv2.MORPH_RECT, (1, kernel_length)
    )

    # A horizontal kernel of (kernel_length X 1), which will help
    # to detect all the horizontal line from the image.
    hori_kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (kernel_length, 1))

    # A kernel of (3 X 3) ones.
    kernel = cv2.getStructuringElement(cv2.MORPH_RECT, (3, 3))

    # Morphological operation to detect verticle lines from an image
    img_temp1 = cv2.erode(img_bin, verticle_kernel, iterations=3)
    verticle_lines_img = cv2.dilate(img_temp1, verticle_kernel, iterations=3)

    # Morphological operation to detect horizontal lines from an image
    img_temp2 = cv2.erode(img_bin, hori_kernel, iterations=3)
    horizontal_lines_img = cv2.dilate(img_temp2, hori_kernel, iterations=3)

    # Weighting parameters, this will decide the quantity of
    # an image to be added to make a new image.
    alpha = 0.5
    beta = 1.0 - alpha

    # This function helps to add two image with specific weight parameter
    # to get a third image as summation of two image.
    img_final_bin = cv2.addWeighted(
        verticle_lines_img, alpha, horizontal_lines_img, beta, 0.0
    )
    img_final_bin = cv2.erode(~img_final_bin, kernel, iterations=2)
    (thresh, img_final_bin) = cv2.threshold(
        img_final_bin, 128, 255, cv2.THRESH_BINARY | cv2.THRESH_OTSU
    )

    mode = cv2.RETR_CCOMP
    method = cv2.CHAIN_APPROX_SIMPLE
    contours, hierarchy = cv2.findContours(img_final_bin, mode, method)

    return contours, hierarchy, img_final_bin


def get_investment_pages(
    pages: List[Image.Image],
):
    """Guess which pages are our pages

    :param pages: List of pages as images
    :return: Tuple of pages
    """

    pg_count = len(pages)
    if pg_count == "6":
        return pages[:3], pages[3:-2], pages[-2]
    cv_image = np.array(pages[3])
    avg_color_per_row = np.average(cv_image, axis=0)
    avg_color = np.average(avg_color_per_row, axis=0)
    if avg_color[0] > 245:
        return pages[:4], pages[4:-2], pages[-2]
    else:
        return pages[:3], pages[3:-2], pages[-2]


def get_text_fields(non_investment_pages):
    """

    :param non_investment_pages:
    :return:
    """
    pg_num = 0
    s1 = []
    for page in non_investment_pages:
        page = page.resize((1653, 2180))
        contours, hierarchy, _ = box_extraction(page)
        i = 0
        while i < len(contours):
            x, y, w, h = cv2.boundingRect(contours[i])
            if w / h > 7 and 1200 > w > 175:
                rect = (x, y, w, h, pg_num, range(y, y + h))
                s1.append(rect)
            i += 1
        pg_num += 1
    return s1


def identify_sections(s1):
    """

    :param s1:
    :return:
    """
    results = load_template()
    df2 = pd.DataFrame(
        {
            "x": [x[0] for x in s1],
            "y": [x[1] for x in s1],
            "w": [x[2] for x in s1],
            "h": [x[3] for x in s1],
            "top": [x[1] + 10 for x in s1],
            "page": [x[4] for x in s1],
        }
    )

    df2 = df2.sort_values(["page", "y", "top"])
    df2["group"] = (
        (
            df2.top.rolling(window=2, min_periods=1).min()
            - df2.y.rolling(window=2, min_periods=1).max()
        )
        < 0
    ).cumsum()
    ndf2 = df2.groupby("group").filter(lambda x: len(x) > 1)
    other_sections = json.loads(ndf2.to_json(orient="table"))
    other_groups = groupby(
        other_sections["data"],
        lambda content: content["group"],
    )
    section = None
    sect_name = None
    last_top = None

    row_index = 0
    for grouping in other_groups:
        col_indx = 0
        groups = list(grouping[1])
        ordered_grp = sorted(groups, key=lambda x: x["x"])
        if section is None:
            section = 1
            sect_name = "Positions"
            # print("\n Section #", section, "\n")
        elif section == 1 and ordered_grp[0]["w"] < 500:
            section = 2
            sect_name = "Agreements"
            # print("\n Section #", section, "\n")
        elif section == 2 and len(ordered_grp) == 3:
            section = 3
            sect_name = "Non-Investment Income"
            # print("\n Section #", section, "\n")
        elif section == 3 and len(ordered_grp) == 2:
            section = 4
            sect_name = "Non Investment Income Spouse"
            # print("\n Section #", section, "\n")
        elif len(ordered_grp) == 5 and section != 5:
            section = 5
            sect_name = "Reimbursements"
            # print("\n Section #", section, "\n")
        elif section == 5 and len(ordered_grp) == 3 and section != 6:
            section = 6
            sect_name = "Gifts"
            # print("\n Section #", section, "\n")
        elif section == 6 and (abs(last_top - ordered_grp[0]["top"]) > 200):
            section = 7
            sect_name = "Liabilities"
            # print("\n Section #", section, "\n")
        last_top = ordered_grp[0]["top"]
        if ordered_grp[0]["x"] > 200:
            continue

        if results["sections"][sect_name]["rows"] == {}:
            row_index = 0
        results["sections"][sect_name]["rows"][row_index] = {}
        for group in sorted(groups, key=lambda x: x["x"]):
            group["coords"] = (
                group["x"],
                group["y"] - 60,
                (group["x"] + group["w"]),
                (group["y"] + group["h"]),
            )
            try:
                column = results["sections"][sect_name]["fields"][col_indx]

                results["sections"][sect_name]["rows"][row_index][
                    column
                ] = group
                results["sections"][sect_name]["empty"] = False
                results["sections"][sect_name]["rows"][row_index][column][
                    "section"
                ] = sect_name
                col_indx += 1
            except IndexError:
                # Remove incomplete rows here
                pass
        row_index += 1
    return results


def extract_section_I_to_VI(
    results: Dict[str, Union[str, int, float, List, Dict]],
    pages: List[Image.Image],
):
    """Extract data from the textfield sections i - vi

    :param results: Dxtracted data
    :param pages: List of images
    :return: Extracted data
    """
    page_is = None
    for k, v in results["sections"].items():
        for x, row in v["rows"].items():
            ocr_key = 1
            for y, column in row.items():
                if page_is == None or page_is != column["page"]:
                    page_is = column["page"]
                    old_page = pages[column["page"]]
                    page = old_page.resize((1653, 2180))

                crop = page.crop(column["coords"])
                if column["section"] == "Liabilities":
                    ocr_key += 1
                    if ocr_key == 4:
                        text = ocr_slice(crop, ocr_key).strip()
                    else:
                        text = ocr_slice(crop, 1).strip()
                else:
                    text = ocr_slice(crop, ocr_key).strip()
                results["sections"][k]["rows"][x][y] = {}
                results["sections"][k]["rows"][x][y]["text"] = text
                results["sections"][k]["rows"][x][y][
                    "is_redacted"
                ] = find_redactions(crop)
    return results


# Deprecated and replaced leaving for now.
def process_page(page, row_count, results, pg_count):
    k = "Investments and Trusts"
    columns = results["sections"]["Investments and Trusts"]["fields"]
    logging.info(f"Extracting Investment Page #{pg_count}")

    data = extract_page(page)
    for row in data:
        if len(row) > len(columns):
            continue
        i = 0
        row_index = str(row_count)
        results["sections"][k]["rows"][row_index] = {}
        for item in row:
            column = columns[i]
            i += 1
            color_coverted = cv2.cvtColor(item, cv2.COLOR_BGR2RGB)
            pil_image = Image.fromarray(color_coverted)
            t = ocr_slice(pil_image, i)
            if column == "A":
                print(clean_stock_names(t))
            results["sections"][k]["rows"][row_index][column] = {
                "text": clean_stock_names(t),
                "is_redacted": find_redactions(pil_image),
                "page_number": pg_count,
            }
        row_count += 1

    print(f"Page #{pg_count} finished.")
    return results


def extract_now(results, k, row, row_index, columns, pg_count):
    """Extract from investments of judicial watch documents

    :param results: Our current extraction results
    :param k: The section
    :param row: the Row we are extracting
    :param row_index: the row count
    :param columns: The available columns
    :param pg_count: The page extraction is on
    :return: Extracted content from row
    """

    i = 0
    results["sections"][k]["rows"][row_index] = {}
    for item in row:
        column = columns[i]
        i += 1
        color_coverted = cv2.cvtColor(item, cv2.COLOR_BGR2RGB)
        pil_image = Image.fromarray(color_coverted)
        t = ocr_slice(pil_image, i)
        if "description" in t.lower() or "assets" in t.lower():
            # If this is a bad PDF we may extract from the addendum.
            # Check if we found it and move along.  Otherwise it could be
            # Bleed over from one page
            return results

        results["sections"][k]["rows"][row_index][column] = {
            "text": clean_stock_names(t),
            "is_redacted": find_redactions(pil_image),
            "page_number": pg_count,
        }
    return results


def extract_section_VII(
    # results: Dict[str : Union[str, list, int, float]],
    results: Dict,
    investment_pages: List,
    pg_count: int,
) -> Dict:
    """Extract content from investment pages on judicial watch documents

    :param results:
    :param investment_pages:
    :return:
    """
    k = "Investments and Trusts"
    columns = results["sections"]["Investments and Trusts"]["fields"]
    row_index = 0
    for page in investment_pages:
        pg_count += 1
        data = extract_page(page)
        for row in data:
            if len(row) > len(columns):
                continue
            results = extract_now(
                results, k, row, row_index, columns, pg_count
            )
            row_index += 1
        # Occasionally the judicial watch documents have multiple documents
        # appended to them.  In this case, we stop processing after we
        # no longer detect multiple rows to process
        logging.info(f"Processing page {pg_count}")
    return results


def process_addendum(addendum_page) -> Dict:
    """Process the addendum sections of the document

    :param addendum_page:
    :return:
    """
    width, height = addendum_page.size
    slice = addendum_page.crop(
        (
            0,
            height * 0.1,
            width,
            height * 0.95,
        )
    )
    return {
        "is_redacted": find_redactions(slice),
        "text": ocr_slice(slice, 1, "Addendum"),
    }


def extract_page(page: Image) -> List[List]:
    """Extract Page rows

    :param page:
    :return:
    """
    max_x = 1653
    max_y = 2180
    current_y, last_y, last_y_hit = 0, 0, 0

    page = page.resize((max_x, max_y))
    pil_image = page.convert("RGB")
    open_cv_image = np.array(pil_image)
    open_cv_image = open_cv_image[:, :, ::-1].copy()
    _, _, processed_image = box_extraction(page)

    data = []
    colors = set()
    first_key = None
    for row in processed_image:
        if first_key == None:
            first_key = sum(row)
        current_y += 1

        if ((float(sum(row)) / first_key)) * 100 < 25:
            if abs(current_y - last_y) > 2:
                horizontal_slice = processed_image[
                    last_y_hit:current_y, 0:max_x
                ]
                horizontal_slice = cv2.cvtColor(
                    horizontal_slice, cv2.COLOR_GRAY2BGR
                )
                h, _, _ = horizontal_slice.shape
                vertical_slices = [
                    horizontal_slice[:, x, y]
                    for x in range(max_x)
                    for y in range(1)
                ]
                s8 = []
                current_x, last_x, last_x_hit = 0, 0, 0
                for vertical_slice in vertical_slices:
                    current_x += 1
                    colors.add(sum(vertical_slice))
                    if sum(vertical_slice) < 1000:
                        if abs(current_x - last_x) > 2:
                            if last_x_hit > 25:
                                s8.append(
                                    open_cv_image[
                                        last_y_hit:current_y,
                                        last_x_hit:current_x,
                                    ]
                                )
                            last_x_hit = current_x
                        last_x = current_x
                if len(s8) >= 10:
                    if len(s8) == 11:
                        s8 = s8[1:]
                    data.append(s8)
                last_y_hit = current_y
            last_y = current_y

    if not list(colors):
        return data
    # Check with this xnumber
    if len(sorted(list(colors))) > 30:
        xnumber = sorted(list(colors))[30]
    else:
        xnumber = sorted(list(colors))[-1]
    if len(data) < 5:
        current_y, last_y, last_y_hit = 0, 0, 0
        data = []
        first_key = None
        for row in processed_image:
            if first_key == None:
                first_key = sum(row)
            current_y += 1

            if ((float(sum(row)) / first_key)) * 100 < 25:
                if abs(current_y - last_y) > 2:
                    horizontal_slice = processed_image[
                        last_y_hit:current_y, 0:max_x
                    ]
                    horizontal_slice = cv2.cvtColor(
                        horizontal_slice, cv2.COLOR_GRAY2BGR
                    )
                    h, _, _ = horizontal_slice.shape
                    vertical_slices = [
                        horizontal_slice[:, x, y]
                        for x in range(max_x)
                        for y in range(1)
                    ]
                    s8 = []
                    current_x, last_x, last_x_hit = 0, 0, 0
                    for vertical_slice in vertical_slices:
                        current_x += 1
                        if sum(vertical_slice) < xnumber:
                            if abs(current_x - last_x) > 2:
                                if last_x_hit > 25:
                                    s8.append(
                                        open_cv_image[
                                            last_y_hit:current_y,
                                            last_x_hit:current_x,
                                        ]
                                    )
                                last_x_hit = current_x
                            last_x = current_x
                    if len(s8) >= 10:
                        if len(s8) == 11:
                            s8 = s8[1:]
                        data.append(s8)
                    last_y_hit = current_y
                last_y = current_y

    return data



import json
import re
from typing import Dict, Union, List
from langdetect import detect
from langdetect.lang_detect_exception import LangDetectException
from numpy import median


def _fine_tune_results(
    results: dict,
) -> Dict[str, Union[str, int, float, List, Dict]]:
    """Clean results by restructuring document

    :param results: Raw data extracted by OCR
    :return: Refined data
    """

    # Remove incomplete rows - and assume artifacts
    # Find rows outside of investments that have missing data and
    # remove them from the results
    remove_tuples = []
    for k, v in results["sections"].items():
        if k == "Investments and Trusts":
            for k1, v1 in v["rows"].items():
                if len(v["fields"]) != len(v1.keys()):
                    remove_tuples.append((k, k1))

    for r in remove_tuples:
        results["sections"][r[0]]["rows"].pop(r[1], None)

    # Reorganize results here
    for k, v in results["sections"].items():
        del v["empty"]
        del v["order"]
        rows = []

        for k1, v1 in v["rows"].items():
            single_row = []
            for k2, v2 in v1.items():
                if v2["text"] is None:
                    v2["text"] = ""
                clean_text = re.sub(
                    r"^(\. )|^([\d]{1,3}(\.|,)? ?)", "", v2["text"]
                )
                single_row.append(clean_text)
            # v1["row_count"] = int(k1) + 1
            if len("".join(single_row)) > 3:
                rows.append(v1)
        v["rows"] = rows

    # Remove junk from - Unused numbers from the first item in a row
    # Excluding Investments and Trusts Section
    for k, sect in results["sections"].items():
        if k == "Investments and Trusts":
            continue
        for row in sect["rows"]:
            for _, field in row.items():
                field["text"] = re.sub(
                    r"^([0-9]{1,2}\.|,) |^(\. )", "", field["text"]
                )
                # Clean up (lL. Board Member -> Board Member)
                field["text"] = re.sub(
                    r"(^\S(?<!U)(\w)?\.? )",
                    "",
                    field["text"],
                    flags=re.IGNORECASE,
                )

                # Capitalize first words in sentences
                if len(field["text"]) > 1:
                    field["text"] = (
                        field["text"][0].upper() + field["text"][1:]
                    )

                # Dates - sometimes get addendum a numerical count
                # (12011 -> 2011; 2.2011 -> 2011)
                year_cleanup_regex = (
                    r"^(1(?P<year1>(20)(0|1)[0-9]))"
                    r"|([1-5]\.(?P<year2>(20)(0|1)[0-9]))"
                )
                m = re.match(year_cleanup_regex, field["text"])
                if m:
                    if m.group("year1"):
                        field["text"] = m.group("year1")
                    elif m.group("year2"):
                        field["text"] = m.group("year2")
                break

    # Cleanup Investments and Trusts B2 field- appears to be dropdown with
    # free text options
    for row in results["sections"]["Investments and Trusts"]["rows"]:
        if "rest" in row["B2"]["text"]:
            row["B2"]["text"] = "Interest"
        if "Dist" in row["B2"]["text"]:
            row["B2"]["text"] = "Distribution"
        if "idend" in row["B2"]["text"]:
            row["B2"]["text"] = "Dividend"

    # Infer values for empty description fields in Investments and Trusts
    # Mark field as inferred value or not
    count = 0
    inv = results["sections"]["Investments and Trusts"]["rows"]
    for i in inv:
        inv[count]["A"]["inferred_value"] = False
        name = i["A"]["text"]
        if len(name) < 4:
            name = ""
        else:
            name = re.sub(r"^(\d|\W)(\S){1,3}\.? ?$", "", name)
        if name == "":
            if i["D1"]["text"] != "":
                inv[count]["A"]["text"] = inv[count - 1]["A"]["text"]
                inv[count]["A"]["inferred_value"] = True

        # Clean up D1 Field/Transaction Type
        # to select Buy Additional or Sold Part
        d1 = inv[count]["D1"]
        if "part)" == d1["text"] or "purt" in d1["text"]:
            d1["text"] = "Sold (part)"
        if "add'l)" == d1["text"] or "d'l)" in d1["text"]:
            d1["text"] = "Buy (add'l)"
        if "ad" in d1["text"]:
            d1["text"] = "Buy (add'l)"

        d2 = inv[count]["D2"]["text"]
        if d2:
            if re.match(r"\d{4}\/(\d{2}|\d{4})", d2):
                d2 = f"{d2[:2]}/{d2[2:]}"
            elif re.match(r"(\d{2})\/\d{4}", d2):
                d2 = f"{d2[:5]}/{d2[5:]}"
            inv[count]["D2"]["text"] = d2

        b2 = inv[count]["B2"]["text"]
        if b2:
            if "lnt/" in b2:
                inv[count]["B2"]["text"] = "Int/Div"
            if (
                "D" == b2
                or "y" == b2
                or "v" == b2
                or "Int/" in b2
                or "Dy" in b2
                or "iv" in b2
                or "iy" in b2
                or "Dv" in b2
                or "vy" == b2
            ):
                inv[count]["B2"]["text"] = "Int/Div"

        count += 1

    # Clean up any incomplete rows or rows containing no english

    for title, sect in results["sections"].items():
        to_remove = set()
        fields = sect["fields"]
        if title != "Investments and Trusts":
            count = 0
            for row in sect["rows"]:
                languages = []

                if len(fields) == len(row.keys()):

                    row_text = [row[field]["text"] for field in fields]
                    lengths = [len(r) for r in row_text]
                    if 0 in lengths:
                        for x in row_text:
                            if x != "":
                                m = median([len(l) for l in x.split(" ")])
                                if m < 2:
                                    to_remove.add(count)

                    for field in fields:
                        if row[field]["is_redacted"]:
                            languages.append("en")
                            continue
                        try:
                            languages.append(detect(row[field]["text"]))
                        except LangDetectException:
                            languages.append("")

                    if "en" not in languages:
                        to_remove.add(count)
                else:
                    to_remove.add(count)
                count += 1
        for i in sorted(list(to_remove))[::-1]:
            results["sections"][title]["rows"].pop(i)

    return json.loads(json.dumps(results).replace("\\u2022", "-1"))



# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import logging
from typing import Dict

import requests
from pdf2image import convert_from_bytes, convert_from_path
from prettytable import PrettyTable

from disclosure_extractor.calculate import estimate_investment_net_worth, color
from disclosure_extractor.data_processing import process_document
from disclosure_extractor.image_processing import (
    extract_contours_from_page,
    CheckboxesNotFound,
)
from disclosure_extractor.judicial_watch_utils import (
    get_investment_pages,
    extract_section_VII,
    extract_section_I_to_VI,
    identify_sections,
    get_text_fields,
    process_addendum,
)
from disclosure_extractor.post_processing import _fine_tune_results


def display_wealth(results: Dict[str, str]) -> None:
    """Print data in nice neat tables

    :param results: Results of extraction
    :return: None
    """
    if "wealth" not in results.keys():
        return

    print("\n", "Wealth", "\n===============================")
    net_worth = results["wealth"]["investment_net_worth"]
    gains = results["wealth"]["income_gains"]
    debts = results["wealth"]["liabilities"]
    salaries = results["wealth"]["salary_income"]

    gross_low, gross_high = [
        color.RED + color.BOLD + str("${:,}".format(x)) + color.END
        if x > 1000000
        else str("${:,}".format(x))
        for x in net_worth
    ]
    gains_low, gains_high = [
        color.RED + color.BOLD + str("${:,}".format(x)) + color.END
        if x > 50000
        else str("${:,}".format(x))
        for x in gains
    ]
    if (net_worth[0] - gains[0]) > 0:
        percent = 100 * gains[0] / (net_worth[0] - gains[0])
        yoy_percent = (
            color.RED
            + color.BOLD
            + str("{:,.2f}%".format(percent))
            + color.END
            if percent > 5
            else str("{:,.2f}%".format(percent))
        )
    else:
        yoy_percent = 0
    debts_low, debts_high = [
        color.RED + color.BOLD + str("${:,}".format(x)) + color.END
        if x > 50000
        else str("${:,}".format(x))
        for x in debts
    ]

    print("Investments Total:".ljust(25), "%s to %s" % (gross_low, gross_high))
    print(
        "Investments gains YOY:".ljust(25),
        "%s to %s" % (gains_low, gains_high),
    )
    print("Percent gains YOY:".ljust(25), "%s" % (yoy_percent))
    print("Debts:".ljust(25), "%s to %s" % (debts_low, debts_high))
    print(
        "Other incomes totaling:".ljust(25),
        "%s" % ("${:,.2f}".format(salaries)),
    )
    print("\n")


def display_table(results: Dict) -> None:
    """Display results in nice neat tables

    :param results: Results from extraction
    :return: None
    """
    for title, sect in results["sections"].items():
        print(title)
        fields = sect["fields"]
        t = PrettyTable(fields)
        for row in sect["rows"]:
            r = [row[field]["text"] for field in sect["fields"]]
            t.add_row(r)
        print(t, end="\n\n")


def process_financial_document(
    file_path=None, url=None, pdf_bytes=None, show_logs=None, resize_pdf=True
):
    """Process Financial Documents

    :param file_path:
    :param url:
    :param pdf_bytes:
    :param show_logs:
    :param resize_pdf:
    :return:
    """
    if show_logs:
        logging.getLogger().setLevel(logging.INFO)

    logging.info("Beginning Extraction of Financial Document")

    if not file_path and not url and not pdf_bytes:
        logging.warning(
            "\n\n--> No file, url or pdf_bytes submitted<--\n--> Exiting early\n"
        )
        return

    if file_path:
        logging.info("Opening PDF document from path")
        pdf_bytes = open(file_path, "rb").read()
    if url:
        logging.info("Downloading PDF from URL")
        pdf_bytes = requests.get(url, stream=True).content

    # Turn the PDF into an array of images
    pages = convert_from_bytes(pdf_bytes)
    page_total = len(pages)
    logging.info("Document is %s pages long" % page_total)
    logging.info("Determining document structure")

    try:
        document_structure = extract_contours_from_page(pages, False)
    except:
        logging.info("Failed")
        return {"success": False, "msg": CheckboxesNotFound}

    if document_structure["found_count"] < 8:
        logging.warning("Failed to extract document structure")
        return {
            "success": False,
            "msg": "Failed to process document properly",
            "checkbox_count_found": document_structure["found_count"],
        }

    logging.info("Extracting content from financial disclosure")
    results = process_document(document_structure, pages)
    results["page_count"] = page_total
    results["pdf_size"] = len(pdf_bytes)
    results["wealth"] = estimate_investment_net_worth(results)
    results["success"] = True

    # Cleanup raw data & update document structure
    cleaned_data = _fine_tune_results(results)
    return cleaned_data


def process_judicial_watch(
    file_path=None, url=None, pdf_bytes=None, show_logs=None
):
    """This is the second and more brute force method for ugly PDFs.

    This method relies upon our own slicing and dicing of the image.
    """
    if show_logs:
        logging.getLogger().setLevel(logging.INFO)

    logging.info("Beginning Extraction of Financial Document")

    if not file_path and not url and not pdf_bytes:
        logging.warning(
            "\n\n--> No file, url or pdf_bytes submitted<--\n--> Exiting early\n"
        )
        return

    if file_path:
        logging.info("Opening PDF document from path")
        pdf_bytes = open(file_path, "rb").read()
    if url:
        logging.info("Downloading PDF from URL")
        pdf_bytes = requests.get(url, stream=True).content

    # Turn the PDF into an array of images
    pages = convert_from_bytes(pdf_bytes, thread_count=10, fmt="jpg", dpi=300)
    page_total = len(pages)
    logging.info("Document is %s pages long" % page_total)

    logging.info("Determining document structure")
    (
        non_investment_pages,
        investment_pages,
        addendum_page,
    ) = get_investment_pages(pages)

    s1 = get_text_fields(non_investment_pages)
    document_data = identify_sections(s1)
    results = extract_section_I_to_VI(document_data, non_investment_pages)

    logging.info("Processing Investments")
    # Process Section VII
    results = extract_section_VII(
        results, investment_pages, len(non_investment_pages)
    )

    # Process Section VIII - Addendum
    addendum_data = process_addendum(addendum_page)
    results["Additional Information or Explanations"] = addendum_data

    # Calculate net worth and mark processing as a success
    results["wealth"] = estimate_investment_net_worth(results)

    # Add final data
    results["page_count"] = page_total
    results["pdf_size"] = len(pdf_bytes)
    results["success"] = True

    # Cleanup raw data & update document structure
    cleaned_data = _fine_tune_results(results)

    return cleaned_data


def extract_financial_document(
    file_path: str = None,
    pdf_bytes: bytes = None,
    show_logs: bool = False,
    resize: bool = False,
) -> Dict:
    """Extract documents with lowered memory footprint


    :param file_path: Location of the PDF to extract
    :param show_logs: Should we show our logs
    :return: Our results of the extracted content
    """

    if show_logs:
        logging.getLogger().setLevel(logging.INFO)
    logging.info("Starting pdf to image conversion")
    if pdf_bytes:
        xpages = convert_from_bytes(
            pdf_bytes, thread_count=10, fmt="jpg", dpi=300
        )
    else:
        xpages = convert_from_path(
            file_path, thread_count=10, fmt="jpg", dpi=300
        )

    logging.info("Document is %s pages long" % len(xpages))
    logging.info("Determining document structure.")

    pages = []
    if resize:
        for page in xpages:
            pg = page.resize((1653, 2180))
            pages.append(pg)

    document_structure = extract_contours_from_page(pages, False)

    if document_structure["found_count"] < 8:
        document_structure = extract_contours_from_page(pages, True)
        # print(document_structure["found_count"])
        if document_structure["found_count"] < 8:

            logging.warning(
                f"Failed to extract document structure {document_structure['found_count']}"
            )
            return {
                "success": False,
                "msg": "Failed to process document properly",
                "checkbox_count_found": document_structure["found_count"],
            }

    logging.info("Extracting content from financial disclosure")
    results = process_document(document_structure, pages)
    results["page_count"] = len(pages)

    results["pdf_size"] = ""
    results["wealth"] = estimate_investment_net_worth(results)
    results["success"] = True
    results["msg"] = ""

    # Cleanup raw data & update document structure
    cleaned_data = _fine_tune_results(results)

    return cleaned_data
